
#include "definitions.bas"
#include "constants.bas"
#include "functions.bas"
#include "strings.bas"
#include "lines.bas"

type TGrid
  as piece array(ONE - 1 to SEVEN + 1, ONE - 1 to SIX + 1)
  declare sub Empty
  declare sub Copy(p() as piece)
  declare function IsFull as boolean
  declare function IsEmpty as boolean
  declare function Insert(p as piece, x as integer) as boolean
  declare function Fill(s as string) as boolean
  declare function LinePoints(l as TLine, p as piece) as integer
  declare function GridPoints(p as piece) as integer
  declare function AsString as string
end type

sub TGrid.Empty
  dim as integer x, y
  for x = ONE - 1 to SEVEN + 1
    for y = ONE - 1 to SIX + 1
      array(x, y) = border
    next y
  next x
  for x = ONE to SEVEN
    for y = ONE to SIX
      array(x, y) = nothing
    next y
  next x
end sub

sub TGrid.Copy(p() as piece)
  for x as integer = ONE to SEVEN
    for y as integer = ONE to SIX
      array(x, y) = p(x, y)
    next y
  next x
end sub

function TGrid.IsFull as boolean
  dim result as boolean = true
  dim x as integer = ONE
  do while (x <= SEVEN) and result
    if array(x, SIX) = nothing then result = false
    x += 1
  loop
  return result
end function

function TGrid.IsEmpty as boolean
  dim result as boolean = true
  dim x as integer = ONE
  do while (x <= SEVEN) and result
    if array(x, SIX) <> nothing then result = false
    x += 1
  loop
  return result
end function

function TGrid.Insert(p as piece, x as integer) as boolean
  dim y as integer = SIX
  dim result as boolean = (array(x, y) = nothing)
  if result then
    do while (array(x, y - 1) = nothing)
      y -= 1
    loop
    array(x, y) = p
  end if
  return result
end function

function TGrid.Fill(s as string) as boolean
  dim result as boolean = IsEmpty and IsValidFormat(s) andalso IsValidSequence(s)
  dim p as piece = white
  if result then
    for i as integer = 0 to len(s) - 1
      Insert(p, AscToInt(s[i]))
      p *= black
    next i
  end if
  return result
end function

function TGrid.LinePoints(l as TLine, p as piece) as integer
  dim as integer x, y, n = 0, o = 0, nm = 0, om = 0, result
  x = l.x1
  y = l.y1
  do while array(x, y) <> border
    if array(x, y) = p then n += 1 else n = 0
    if array(x, y) <> black * p then o += 1 else o = 0
    if n > nm then nm = n
    if o > om then om = o
    x += l.dx
    y += l.dy 
  loop
  if om < 4 then result = 0 else result = nm
  return result
end function

function TGrid.GridPoints(p as piece) as integer
  dim as integer i, j, k = 0
  for i = 0 to 24
    j = LinePoints(lines(i), p)
	if j > k then k = j
  next i
  return k
end function

function TGrid.AsString as string
  dim result as string = "   a b c d e f g" & chr(13, 10)
  for y as integer = SIX to ONE step - 1
    result = result & str(y) & " "
    for x as integer = ONE to SEVEN
      select case array(x, y)
        case nothing
          result = result & " ."
        case white
          result = result & " X"
        case black
          result = result & " O"
      end select
    next x
    result = result & chr(13, 10)
  next y
  return result
end function
