
'#define usecairo
#ifdef usecairo
#include "cairo.bi"
#endif

#include ".\libraries\mdlanguage\mdlanguage.bi"
#include ".\libraries\iniobject\inifile.bi"
#undef true
#undef false

#include ".\libraries\sgui\sgui.bas"
#include ".\libraries\sgui\checkmarkgadget.bas"
#include ".\libraries\sgui\simplegadget.bas"
#undef black
#undef white

#include ".\colors.bas"

extern "C" lib ".\engine"
  declare function Answer stdcall alias "Answer" (byval s as string) as integer
end extern

extern "C" lib ".\veleng"
  declare function VelenaAnswer alias "Answer" (byval as zstring ptr) as integer
end extern

#include "graph.bas"
#include "grid.bas"

using inifobj
using sGUI

declare sub Display(as string)
declare function EmptyCell(as tGrid, as integer) as integer
declare sub ReadIniFile(byref as integer, byref as integer, byref as string)
declare function Exists(file as string) as integer
declare sub InsertAndCheckGameState
declare sub Restart

dim as integer CPW, CPB
dim language as string
ReadIniFile(CPW, CPB, language)
LoadCustomLanguage(default, language, "properties\")

dim as string s1
dim shared as string s4, s5, s6, s8, s9
s1 = GetTranslation(default, "s1")
s4 = GetTranslation(default, "s4")
s5 = GetTranslation(default, "s5")
s6 = GetTranslation(default, "s6")
s8 = GetTranslation(default, "s8")
s9 = GetTranslation(default, "s9")
if language = "de" then
  s6 = "Wei" & chr(225) & " gewinnt"
  s8 = "Wei" & chr(225) & " am Zug"
end if

const SW = 8 * 48, SH = 10 * 48
screenres SW, SH, 32
width SW \ 8, SH \ 16
windowtitle s1
InitGFX

dim shared v as TGraph
v.Initialize(48, 20, 0, 48, gridColor, noPieceColor, whitePieceColor, blackPieceColor)
v.DrawGrid

dim shared as EventHandle ptr event
event = new EventHandle

dim as Gadget ptr b1, b2, b3, b4, b5, b6, b7
b1 = AddSimpleGadget(event, 0 * 48 + 4 + 24, 9 * 48 + 12, 40, 24, "")
b2 = AddSimpleGadget(event, 1 * 48 + 4 + 24, 9 * 48 + 12, 40, 24, "")
b3 = AddSimpleGadget(event, 2 * 48 + 4 + 24, 9 * 48 + 12, 40, 24, "")
b4 = AddSimpleGadget(event, 3 * 48 + 4 + 24, 9 * 48 + 12, 40, 24, "")
b5 = AddSimpleGadget(event, 4 * 48 + 4 + 24, 9 * 48 + 12, 40, 24, "")
b6 = AddSimpleGadget(event, 5 * 48 + 4 + 24, 9 * 48 + 12, 40, 24, "")
b7 = AddSimpleGadget(event, 6 * 48 + 4 + 24, 9 * 48 + 12, 40, 24, "")
GadgetOn(b1, b7)

dim shared g as tGrid
dim shared p as piece
dim shared as integer x, y
dim shared h as string
dim shared as integer whitePoints, blackPoints
dim shared gameOver as boolean

g.Empty
p = pwhite
x = 0
y = 0
h = ""
whitePoints = 0
blackPoints = 0
gameOver = false

Display(iif(p = pwhite, s8, s9))

do
  event->xSleep(1)
  if event->GADGETMESSAGE then
    select case event->GADGETMESSAGE
      case b1 : x = 1
      case b2 : x = 2
      case b3 : x = 3
      case b4 : x = 4
      case b5 : x = 5
      case b6 : x = 6
      case b7 : x = 7
    end select
    if gameOver then
      Restart
    else
      y = EmptyCell(g, x)
      if (x > 0) andalso (y > 0) then
        InsertAndCheckGameState
      end if
    end if
  end if
  if ((p = pwhite) and CPW) or ((p = pblack) and CPB) then
    if p = pwhite then
      x = iif(CPW = 1, Answer(h), VelenaAnswer("c" & h & "0"))
    else
      x = iif(CPB = 1, Answer(h), VelenaAnswer("c" & h & "0"))
    end if
    y = EmptyCell(g, x)
    InsertAndCheckGameState
  end if
loop until event->EXITEVENT

delete event

#ifdef usecairo
cairo_destroy(v.cr)
#endif
end

sub Display(s as string)
  FillA(24, 12, 7 * 48, 24, windowColor, 1)
  draw string((7 * 48 - 8 * len(s)) \ 2 + 24, 12 + 4), s, textColor
end sub

function EmptyCell(aGrid as tGrid, aCol as integer) as integer
  dim as integer result = 0
  dim as integer y = 0
  while (y < 6) and (result = 0)
    y += 1
    if aGrid.array(aCol, y) = pnothing then result = y
  wend
  function = result
end function

sub ReadIniFile(byref aCPW as integer, byref aCPB as integer, byref aLanguage as string)
  dim myIni as iniobj = iniobj("options.ini")
  select case LCase(myIni.ReadString("players", "white", ""))
    case "engine"
      aCPW = 1
    case "velena"
      aCPW = 2
    case else
      aCPW = 0
  end select
  select case LCase(myIni.ReadString("players", "black", ""))
    case "engine"
      aCPB = 1
    case "velena"
      aCPB = 2
    case else
      aCPB = 0
  end select
  aLanguage = LCase(myIni.ReadString("interface", "language", ""))
  select case aLanguage
    case "de", "en", "fr"
      rem
    case else
      aLanguage = "en"
  end select
end sub

function Exists(file as string) as integer
  return (len(dir(file)) > 0) orelse (len(dir(file & ".exe")) > 0)
end function

sub InsertAndCheckGameState
  dim s as string
  v.InsertPiece(x, y, iif(p=pwhite, whitePieceColor, blackPieceColor))
  g.Insert(p, x)
  p *= pblack
  h += str(x)
  x = 0
  whitePoints = g.GridPoints(pwhite)
  blackPoints = g.GridPoints(pblack)
  if (whitePoints >= 4) or (blackPoints >= 4) or g.IsFull then
    if g.IsFull then s = s4
    if blackPoints >= 4 then s = s5
    if whitePoints >= 4 then s = s6
    gameOver = true
  else
    s = iif(p = pwhite, s8, s9)
  end if
  Display(s)
end sub

sub Restart
  p = pwhite
  h = ""
  whitePoints = 0
  blackPoints = 0
  g.Empty
  v.DrawGrid
  gameOver = false
  Display(iif(p = pwhite, s8, s9))
end sub
