
#ifdef usecairo

#define SetSourceColor(cairo, colour) cairo_set_source_rgb(cairo, (colour and &hFF0000) / &hFF0000, (colour and &h00FF00) / &h00FF00, (colour and &h0000FF) / &h0000FF)

const PI = 3.141592654

type TGraph
  as integer w1, w2, l1, t1, c1, c2, c3, c4
  as cairo_surface_t ptr surface
  as cairo_t ptr cr
  declare sub Initialize(_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer)
  declare sub DrawPiece(as integer, as integer, as integer)
  declare sub DrawGrid
  declare sub InsertPiece(as integer, as integer, as integer)
end type

sub TGraph.Initialize(_
  aw1 as integer,_
  aw2 as integer,_
  al1 as integer,_
  at1 as integer,_
  ac1 as integer,_
  ac2 as integer,_
  ac3 as integer,_
  ac4 as integer)
  w1 = aw1
  w2 = aw2
  l1 = al1
  t1 = at1
  c1 = ac1
  c2 = ac2
  c3 = ac3
  c4 = ac4
  surface = cairo_image_surface_create_for_data(_
    screenptr(), _
    CAIRO_FORMAT_ARGB32, _
    8 * w1, _
    10 * w1, _
    8 * w1 * sizeof(integer) _
 )
  cr = cairo_create(surface)
  SetSourceColor(cr, gridColor)
  cairo_paint(cr)
  line(0, 0)-(8 * w1, 1 * w1), blackPieceColor, bf
  line(0, 9 * 48)-(8 * w1, 10 * w1), blackPieceColor, bf
end sub

sub TGraph.DrawPiece(x as integer, y as integer, c as integer)
  dim as integer x1 = w1 * x + l1, y1 = t1 + 8 * w1 - (w1 * y)
  line(x1 - w2, y1 - w2)-(x1 + w2, y1 + w2), c1, bf
  SetSourceColor(cr, c)
  cairo_arc(cr, x1, y1, w2, 0, 2 * PI)
  cairo_fill(cr)
end sub

sub TGraph.DrawGrid
  for x as integer = 1 to 7
    for y as integer = 1 to 6
      DrawPiece(x, y, c2)
    next y
  next x
end sub

sub TGraph.InsertPiece(x as integer, y as integer, c as integer)
  const pause = 40
  for y1 as integer = 7 to y step -1
    screenlock()
    if y1 < 7 then
      if y1 = 6 then
        DrawPiece(x, 7, c1)
      else
        DrawPiece(x, y1 + 1, c2)
      end if
    end if
    DrawPiece(x, y1, c)
    screenunlock()
    sleep pause
  next y1
end sub

#else

type TGraph
  as integer w1, w2, l1, t1, c1, c2, c3, c4
  declare sub Initialize(_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer,_
    as integer)
  declare sub DrawPiece(as integer, as integer, as integer)
  declare sub DrawGrid
  declare sub InsertPiece(as integer, as integer, as integer)
end type

sub TGraph.Initialize(_
  aw1 as integer,_
  aw2 as integer,_
  al1 as integer,_
  at1 as integer,_
  ac1 as integer,_
  ac2 as integer,_
  ac3 as integer,_
  ac4 as integer)
  w1 = aw1
  w2 = aw2
  l1 = al1
  t1 = at1
  c1 = ac1
  c2 = ac2
  c3 = ac3
  c4 = ac4
end sub

sub TGraph.DrawPiece(x as integer, y as integer, c as integer)
  dim as integer x1 = w1 * x + l1, y1 = t1 + 8 * w1 - (w1 * y)
  line(x1 - w2, y1 - w2)-(x1 + w2, y1 + w2), c, bf
end sub

sub TGraph.DrawGrid
  line(l1, t1)-(l1 + 8 * w1, t1 + 8 * w1), c1, bf
  for x as integer = 1 to 7
    for y as integer = 1 to 6
      DrawPiece(x, y, c2)
    next y
  next x
end sub

sub TGraph.InsertPiece(x as integer, y as integer, c as integer)
  const pause = 40
  for y1 as integer = 7 to y step -1
    if y1 < 7 then
      if y1 = 6 then
        DrawPiece(x, 7, c1)
      else
        DrawPiece(x, y1 + 1, c2)
      end if
    end if
    DrawPiece(x, y1, c)
    sleep pause
  next y1
end sub

#endif
